-------------------------------------------------------------------------------
-- Adapplet animtext                                                         --
-- (C) Copyright 1998 ADALOG                                                 --
-- Author: J-P. Rosen                                                        --
--                                                                           --
-- Provides parameterizable animation effects to a text.                     --
-- The text bounces in its box, and its size can change.                     --
--                                                                           --
-- Applet parameters:                                                        --
-- Name      Default   Usage                                                 --
-- ----      -------   -----                                                 --
-- text        "?"     Text to be displayed.                                 --
-- period      20      Period to modify text position, in ms.                --
-- stepx       0       Step by which text is moved horizontally. If 0, text  --
--                     is centered horizontally.                             --
-- stepy       0       Step by which text is moved vertically. If 0, text    --
--                     is centered vertically.                               --
-- minsize     0       Smallest size of text, in points.                     --
-- maxsize     0       Biggest  size of text, in points.                     --
--                     If either size is 0, text has fixed size of the other --
--                     size. If both are 0, text has default browser size.   --
-- sizeperiod  1       Divider of period at which text size is changed.      --
--                     e.g. if sizeperiod=4, size is changed every 4 moves.  --
--                                                                           --
-- Rights to use, distribute or modify this adapplet in any way is hereby    --
-- granted, provided this header is kept unchanged in all versions.          --
-- Additionnal headers may be added. If you make a valuable addition,        --
-- please keep us informed by sending a message to rosen.adalog@wanadoo.fr   --
--                                                                           --
-- ADALOG is providing training, consultancy and expertise in Ada and        --
-- related software engineering techniques. For more info about our services:--
-- ADALOG                   Tel: +33 1 41 24 31 40                           --
-- 19-21 rue du 8 mai 1945  Fax: +33 1 41 24 07 36                           --
-- 94110 ARCUEIL            E-m: info@adalog.fr                              --
-- FRANCE                   URL: http://www.adalog.fr/                       --
--                                                                           --
-- This adapplet is distributed in the hope that it will be useful, but      --
-- WITHOUT ANY WARRANTY;  without even the  implied warranty of              --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      --
-------------------------------------------------------------------------------
with Java.Applet.Applet, Java.Lang.String, Java.Awt.Graphics;
with Java.Awt.Component;
use Java.Awt.Graphics, Java.Applet.Applet;
with Periodic_Updater;
package Animtext is
   pragma Elaborate_Body;
      
   type AnimText_Obj is new Applet_Obj with private;
   type AnimText_Ptr is access all AnimText_Obj'Class;

	procedure Init   (This : access AnimText_Obj);
	procedure Start  (This : access AnimText_Obj);       
	procedure Stop   (This : access AnimText_Obj);
   procedure Paint  (This : access AnimText_Obj;  G : Graphics_Ptr);
   procedure Update (This : access AnimText_Obj;  G : Graphics_Ptr);
       
private
   use Java.Lang.String, Periodic_Updater;
   
	type AnimText_Obj is new Applet_Obj with 
        record
           Updater: Periodic_Updater_PTR;
           X      : Integer := 5;
           Step_X : Integer := 1; 
              
           Y      : Integer := 20;
           Step_Y : Integer := 1;
              
           Min_Size    : Integer;
           Max_Size    : Integer;
           Step_Size   : Integer;
           Size_Period : Integer;
           Size_Counter: Natural;
              
           Text   : String_Ptr;
        end record;
end AnimText;

with Java.Awt.Dimension, Java.Awt.Font, Java.Awt.FontMetrics, Java.Awt.Color;

--with Debug; use Debug;
with Interfaces.Java; use Interfaces.Java;
with Defaulted_Parameters;
package body Animtext is

   -- Utility to compute the size of a text
   type Text_Box is 
      record
         Width : Natural;
         Height: Natural;
      end record;
   function Get_BoundingBox (Obj : Applet_Ptr; Text : String_PTR) return Text_Box is
      use Java.Awt.FontMetrics, Java.Awt.Font;
      
      This_Metric : constant FontMetrics_PTR := GetFontMetrics (GetGraphics(Obj));
   begin
      return (Width  => StringWidth(This_Metric, Text), 
              Height => GetHeight(This_Metric));
   end Get_BoundingBox;
         
   procedure Init (This : access AnimText_Obj) is
      use Defaulted_Parameters;
   	use Java.Awt.Font, Java.Awt.Color;
      Bounding_Box : Text_Box;
   begin
      SetBackground (This, White);
      
		This.Step_X := GetParameter (This, "stepx", 0);
		This.Step_Y := GetParameter (This, "stepy", 0);
		             
		This.Min_Size    := GetParameter (This, "minsize", 0);
		This.Max_Size    := GetParameter (This, "maxsize", 0);
		This.Step_Size   := 1;
		This.Size_Period := GetParameter (This, "sizeperiod", 1);
      This.Size_Counter:= 0;

		This.Text := GetParameter (This, "text", "?");
                    
      -- Sanity checks
      if This.Size_Period <= 0 then
         This.Size_Period := 1;
      end if;
      
      if This.Min_Size <= 0 or This.Max_Size <= 0 then
         This.Step_Size   := 0;
         This.Size_Period := Integer'Last;
         This.Max_Size    := Integer'Max (0, Integer'Max (This.Min_Size, This.Max_Size));
         if This.Max_Size = 0 then
            This.Max_Size := GetSize(GetFont(This));
         end if;
      end if;

      SetFont (This, New_Font (+"Times", PLAIN, This.Max_Size));
      Bounding_Box := Get_BoundingBox (This.all'access, This.Text);
      -- Text is initially centered
      This.X := (Size(This).Width  - Bounding_Box.Width)/2;
		This.Y := (Size(This).Height + Bounding_Box.Height)/2;
   end Init;
     
   procedure Start (This : access AnimText_Obj) is
      use Defaulted_Parameters;
   	use Java.Awt.Font, Java.Awt.Component;
   begin   
      if This.Updater = null then
         This.Updater := New_Periodic_Updater (Long_Integer(GetParameter (This, "period", 20)), 
                                              Component_Ptr(This));
         Start (This.Updater);
      end if;
   end Start;
       
	procedure Stop  (This : access AnimText_Obj) is
   begin
      if This.Updater /= null then
         Stop (This.Updater);
         This.Updater := null;
      end if;
	end Stop;
  
   procedure Paint(This : access AnimText_Obj;  G : Graphics_Ptr) is
   begin
      DrawString (G, This.Text, This.X, This.Y);
   end Paint;

    procedure Update (This  : access AnimText_Obj;  G : Graphics_Ptr) is
      use Java.Awt.Dimension, Java.Awt.Font;

      This_Font      : constant Font_PTR := GetFont(This);
      This_FontSize  : constant Integer  := GetSize(This_Font);
      This_Dimension : constant Dimension_Ptr := Size (This);
      Bounding_Box   : Text_Box;
      New_X, New_Y   : Integer;
   begin
      This.Size_Counter := (This.Size_Counter + 1) mod This.Size_Period;
      if This.Size_Counter = 0 then
         if This_FontSize >= This.Max_Size or else This_FontSize <= This.Min_Size then
            This.Step_Size := -This.Step_Size;
         end if;
         SetFont (This, New_Font(GetFamily (This_Font), GetStyle (This_Font), This_FontSize + This.Step_Size));
      end if;
              
      Bounding_Box := Get_BoundingBox (This.all'access, This.Text);
      
      if This.Step_X = 0 or else Bounding_Box.Width > This_Dimension.Width then
         --Text is fixed or too wide, center in window
         New_X := (Size(This).Width  - Bounding_Box.Width)/2;
      else       
         New_X := This.X + This.Step_X;
         if New_X + Bounding_Box.Width >= This_Dimension.Width
	      then
	         New_X       := This_Dimension.Width - Bounding_Box.Width;
	         This.Step_X := - abs This.Step_X;
	      elsif New_X < 0 then
	         New_X       := 0;
	         This.Step_X := abs This.Step_X;
         end if;
      end if;   
      
      if This.Step_Y = 0 or else Bounding_Box.Height > This_Dimension.Height then
         --Text is fixed or too high, center in window
		   New_Y := (Size(This).Height + Bounding_Box.Height)/2;
      else
         New_Y := This.Y + This.Step_Y;
	      if New_Y >= This_Dimension.Height then
			   New_Y := This_Dimension.Height -1;
	         This.Step_Y := - abs This.Step_Y;
	      elsif New_Y < Bounding_Box.Height - 1 then
	         New_Y       := Bounding_Box.Height - 1;
	         This.Step_Y := abs This.Step_Y;
	      end if;
      end if;
      This.X := New_X;
      This.Y := New_Y;
      
      Update (Applet_Obj(This.all)'access, G);
   end Update;
   
end AnimText;
